/*
 * Copyright (c) 2007-2010 Sonatype, Inc. All rights reserved.
 *
 * "Sonatype" is a trademark of Sonatype, Inc.
 */

package org.sonatype.tycho.p2.publisher.plugin;

import java.io.File;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;

import org.apache.maven.model.Repository;
import org.apache.maven.plugin.MojoExecutionException;
import org.codehaus.plexus.util.StringUtils;
import org.sonatype.tycho.p2.bridge.client.P2DirectorFactory;
import org.sonatype.tycho.p2.bridge.client.P2ProfileRegistryFactory;
import org.sonatype.tycho.p2.bridge.facade.P2Director;
import org.sonatype.tycho.p2.bridge.facade.P2ProfileRegistry;
import org.sonatype.tycho.p2.bridge.facade.ProfileTimestamp;
import org.sonatype.tycho.p2.eclipse.bridge.EclipseInstance;
import org.sonatype.tycho.p2.publisher.P2Tasks;
import org.sonatype.tycho.p2.publisher.PublisherResult;
import org.sonatype.tycho.p2.publisher.model.InstallableUnit;

/**
 * @goal assemble
 * @phase process-resources
 * @requiresDependencyResolution runtime
 */
public class AssembleMojo
    extends PublishMojo
{

    /**
     * @parameter expression="${project.build.directory}/tycho-p2/p2-install-folder"
     * @required
     */
    protected File installDirectory;

    /**
     * @parameter
     */
    protected String description;

    /**
     * @component
     */
    private P2DirectorFactory p2DirectorFactory;

    /**
     * @component
     */
    private P2ProfileRegistryFactory p2ProfileRegistryFactory;

    @Override
    protected void doWithEclipse( final EclipseInstance eclipse )
        throws MojoExecutionException
    {
        final PublisherResult publisherResult = executePublisher( eclipse );

        if ( StringUtils.isBlank( description ) )
        {
            final InstallableUnit rootIU = publisherResult.getRootInstallableUnits().get( 0 );
            description = String.format( "Runtime build of %s", rootIU.getVersion() );
        }

        final String[] directorParameters =
            P2Tasks.getDirectorParameters( installDirectory, updateSiteDirectory, publisherResult.getRootInstallableUnits(), getRepositoriesWithLayout( "p2" ), description );

        final P2Director p2Director = p2DirectorFactory.create( eclipse );

        getLog().debug( String.format( "Running P2 director with parameters: %s", Arrays.toString( directorParameters ) ) );
        p2Director.run( new PluginLogProxy( getLog() ), directorParameters );

        final P2ProfileRegistry p2ProfileRegistry = p2ProfileRegistryFactory.create( eclipse );

        // P2 creates an empty profile as first profile so we remove it
        final ProfileTimestamp[] profileTimestamps =
            p2ProfileRegistry.getProfileTimestamps( installDirectory, P2Tasks.P2_PROFILE );
        if ( profileTimestamps != null && profileTimestamps.length > 1 )
        {
            p2ProfileRegistry.removeProfile( installDirectory, P2Tasks.P2_PROFILE, profileTimestamps[0].getTimestamp() );
        }
    }

    private Collection<String> getRepositoriesWithLayout( final String layout )
    {
        final HashSet<String> repos = new HashSet<String>();
        final List<Repository> repositories = project.getRepositories();
        if ( repositories != null )
        {
            for ( final Repository repository : repositories )
            {
                if ( layout.equalsIgnoreCase( repository.getLayout() ) )
                {
                    repos.add( repository.getUrl() );
                }
            }
        }
        return repos;
    }

}
